/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.iteration;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Iteration mit einer Bedingung, die bei jedem Schritt geprüft wird.<br>
 * Die Verwendung des Bedingungsiterators entspricht einer WHILE-Schleife.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class BedingungsIterator implements IterationsBedingung {

	/**
	 * der Ausdruck der die Bedingung beschreibt.
	 */
	private final Ausdruck bedingung;

	/**
	 * Konstruktor.
	 *
	 * @param bedingung
	 *            die auszuwertende Bedingung
	 */
	public BedingungsIterator(final Ausdruck bedingung) {
		this.bedingung = bedingung;
	}

	/**
	 * Die Funktion wertet den Bedingungsausdruck aus und liefert den Wert
	 * "wahr" bzw. "falsch" für die Gültigkeit der Bedingung. Ein nicht
	 * behandelter Fehler bei der Auswertung der Bedingung wird weitergereicht.
	 *
	 * @param kontext
	 *            der Kontext, in dem der Ausdruck ausgewertet wird
	 * @return einen {@link LogischerWert}, der den Wert der Bedingung darstellt
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public LogischerWert interpret(final Kontext kontext) {
		return (LogischerWert) bedingung.interpret(kontext);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Der Bedingungsiterator hat keine Laufvariable. Die Funktion liefert daher
	 * immer den Wert <i>null</i>.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#getLaufVariable()
	 */
	@Override
	public String getLaufVariable() {
		return null;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Ein nicht behandelter Fehler bei der Auswertung der Bedingung wird
	 * weitergereicht.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#isGueltig(de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext)
	 */
	@Override
	public boolean isGueltig(final SkriptKontext kontext) {
		return interpret(kontext).getBoolWert();
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.anweisung.iteration.IterationsBedingung#reset()
	 */
	@Override
	public void reset() {
		// keine Aktionen erforderlich
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(bedingung);
		return liste;
	}
}
