/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * eine Bedingung für die Auswahl von Elementen aus einem Container.<br>
 * Die Klasse enthält einen Attributnamen und einen Vergelichswert. Die
 * Bedingung ist gültig, wenn das über den namen identifizierte Attribut eines
 * Elements mit dem Wert der Selektionsbedingung übereinstimmt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SelektionsBedingung {

	/**
	 * die Liste der Bedingungen.
	 */
	private final Map<String, Object> bedingungen = new HashMap<>();

	/**
	 * fügt der Bedingung einen Vergleichsausdruck hinzu.
	 *
	 * @param name
	 *            der Name des Elements
	 * @param object
	 *            die Daten des Elements
	 */
	public void addItem(final String name, final Object object) {
		bedingungen.put(name, object);
	}

	/**
	 * ermittelt die für die Bedingung gültigen Objekte aus dem übergebenen
	 * Container.<br>
	 * Die Funktion vergleicht alle Elemente in dem übergebenen Container mit
	 * den in den einzelnen Vergelichsbedingungen enthaltenen Objekten. Bei
	 * Gleichhit werden diese in die Ergebnismenge übernommen.
	 *
	 * @param container
	 *            die Elemente aus denen eine Auswahl erfolgen soll
	 * @param single
	 *            es soll nur ein Element ermittelt werden, d.h. die Suche kann
	 *            nach dem ersten passenden Element abgebrochen werden
	 * @return das ermittelte Objekt bzw. eine Liste von Objekten
	 */
	public Object select(final UdaContainer container, final boolean single) {
		Object result = null;

		if (!single) {
			result = new Liste();
		}

		for (final Object obj : container.getElemente()) {
			if (obj instanceof Struktur) {
				int matches = 0;
				for (final Entry<String, Object> bedingung : bedingungen.entrySet()) {
					final Object data = ((Struktur) obj).getStrukturElement(bedingung.getKey());
					if (bedingung.getValue().equals(data)) {
						matches++;
					} else {
						break;
					}
				}
				if (matches == bedingungen.size()) {
					if (single) {
						result = obj;
						break;
					}
					if (result != null) {
						((Liste) result).anhaengen(-1, obj);
					}
				}
			}
		}

		if (result == null) {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.SELEKTION,
					"Kein Objekt für die Auswahlkriterien gefunden");
		}
		return result;
	}
}
