/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import java.text.MessageFormat;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.TimeAttributeType;
import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.KonfigurationsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitDauer;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.ibv.uda.uda.UdaMessage;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Repräsentation eines Datenverteiler-<i>Data</i>-Objekts in einem
 * Uda-Skript.<br>
 * Die Attribute des Objekts hängen von der Definition in der Konfiguration des
 * Datenverteilers ab, d.h. der Zugriff kann über den Attributnamen erfolgen.
 * Das Objekt selbst erlaubt keinen Zugriff auf die Meta-Daten. Der Zugriff ist
 * nur über Funktionen möglich:
 * <ul>
 * <li>typ(): liefert den Attributtyp des Objektes. Über den Attributtyp können
 * zusätzliche Informationen über die empfangenen Daten aus der Konfiguration
 * ermittelt werden.</li>
 * <li>name(): liefert den Namen des Attributs</li>
 * <li>wert(): liefert den Wert des Objekts (Details siehe:
 * {@link DavDatenObjekt#getWertObjekt()})</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DavDatenObjekt extends DavDaten implements Kopierbar {

	/**
	 * Logger für Debugausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstruktor.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param objekt
	 *            das zu Grunde liegende Datenobjekt
	 */
	protected DavDatenObjekt(final ClientDavInterface verbindung, final Data objekt) {
		super(verbindung, objekt);
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public Kopierbar erzeugeKopie() {
		Data daten = null;
		if (getObjekt() != null) {
			daten = getObjekt().createModifiableCopy();
		}
		return new DavDatenObjekt(getVerbindung(), daten);
	}

	/**
	 * liefert die Einheit des Objekts.
	 *
	 * @return die Einheit
	 */
	public Object getEinheit() {
		final DavDaten type = DavDaten.erzeugeObjekt(getVerbindung(), getObjekt().getAttributeType());
		return type.getStrukturElement("einheit");
	}

	/**
	 * liefert den Wertebereich des Objekts.
	 *
	 * @return den Wertebereich oder den Nichtwert, wenn keiner existiert
	 */
	public Object getInfo() {
		final DavDaten type = DavDaten.erzeugeObjekt(getVerbindung(), getObjekt().getAttributeType());
		return type.getStrukturElement("info");
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.DavDaten#getObjekt()
	 */
	@Override
	public Data getObjekt() {
		return (Data) super.getObjekt();
	}

	/**
	 * liefert die Statusliste eines Integerattributs.
	 *
	 * @return die Liste der definierten Statuswerte
	 */
	public Object getStatusListe() {
		final DavDaten type = DavDaten.erzeugeObjekt(getVerbindung(), getObjekt().getAttributeType());
		return type.getStrukturElement("statusliste");
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;
		final Data objekt = getObjekt();
		try {
			result = DavDaten.erzeugeObjekt(getVerbindung(), objekt.getItem(name));
		} catch (final UnsupportedOperationException e) {
			throw new KonfigurationsFehler(UdaFehlerSubtyp.ATTRIBUT, e.getMessage());
		} catch (final IllegalArgumentException e) {
			throw new KonfigurationsFehler(UdaFehlerSubtyp.ATTRIBUT, e.getMessage());
		}
		return result;
	}

	/**
	 * liefert den Wertebereich des Objekts.
	 *
	 * @return den Wertebereich oder den Nichtwert, wenn keiner existiert
	 */
	public Object getWertebereich() {
		final DavDaten type = DavDaten.erzeugeObjekt(getVerbindung(), getObjekt().getAttributeType());
		return type.getStrukturElement("wertebereich");
	}

	/**
	 * liefert ein einem Attribut entsprechendes Objekt, das innerhalb des
	 * Uda-Interpreters verarbeitet werden kann.
	 *
	 * @return das Objekt, das den Wert repräsentiert
	 */
	Object getWertObjekt() {
		final Data data = getObjekt();
		Object ergebnis = null;
		final AttributeType type = data.getAttributeType();
		if (type instanceof AttributeListDefinition) {
			ergebnis = this;
		} else if (type instanceof DoubleAttributeType) {
			ergebnis = Double.valueOf(data.asScaledValue().doubleValue());
		} else if (type instanceof IntegerAttributeType) {
			final IntegerValueRange range = ((IntegerAttributeType) type).getRange();
			if ((range != null) && (range.getConversionFactor() != 1)) {
				ergebnis = Long.valueOf(data.asScaledValue().longValue());
			} else {
				ergebnis = Long.valueOf(data.asUnscaledValue().longValue());
			}
		} else if (type instanceof ReferenceAttributeType) {
			ergebnis = DavDaten.erzeugeObjekt(getVerbindung(), data.asReferenceValue().getSystemObject());
		} else if (type instanceof StringAttributeType) {
			ergebnis = data.asTextValue();
		} else if (type instanceof TimeAttributeType) {
			ergebnis = new Zeitstempel(data.asTimeValue().getMillis());
		} else {
			throw new KonfigurationsFehler(UdaFehlerSubtyp.OBJEKT,
					"Typ wird nicht unterstützt: " + type.getClass().getName());
		}
		if (ergebnis == null) {
			ergebnis = NichtWert.NICHTWERT;
		}
		return ergebnis;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.dav.DavDaten#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		final Data data = getObjekt();

		final Data item = data.getItem(name);
		final AttributeType type = item.getAttributeType();

		if (type instanceof TimeAttributeType) {
			if (daten instanceof Number) {
				item.asTimeValue().setMillis(((Number) daten).longValue());
			} else if (daten instanceof Zeitstempel) {
				item.asTimeValue().setMillis(((Zeitstempel) daten).getMilliSekunden());
			} else if (daten instanceof ZeitDauer) {
				item.asTimeValue().setMillis(((ZeitDauer) daten).getMilliSekunden());
			} else {
				LogTools.log(DavDatenObjekt.LOGGER, UdaMessage.WarningSetzenZeitAttribut, daten);
				throw new IllegalArgumentException(
						MessageFormat.format(UdaMessage.WarningSetzenZeitAttribut.toString(), daten));
			}
		}
		if (type instanceof ReferenceAttributeType) {
			if (daten instanceof SystemObjectObjekt) {
				data.getReferenceValue(name).setSystemObject(((SystemObjectObjekt) daten).getObjekt());
			} else if (daten instanceof NichtWert) {
				data.getReferenceValue(name).setSystemObject(null);
			}
		} else if ((daten instanceof Double) || (daten instanceof Float)) {

			if (type instanceof IntegerAttributeType) {
				final IntegerValueRange range = ((IntegerAttributeType) type).getRange();
				if ((range != null) && (range.getConversionFactor() != 1)) {
					data.getScaledValue(name).set(((Number) daten).doubleValue());
				} else {
					data.getUnscaledValue(name).set(((Number) daten).doubleValue());
				}
			} else {
				data.getUnscaledValue(name).set(((Number) daten).doubleValue());
			}
		} else if (daten instanceof Number) {
			if (type instanceof IntegerAttributeType) {
				final IntegerValueRange range = ((IntegerAttributeType) type).getRange();
				if ((range != null) && (range.getConversionFactor() != 1)) {
					data.getScaledValue(name).set(((Number) daten).longValue());
				} else {
					data.getUnscaledValue(name).set(((Number) daten).longValue());
				}
			} else {
				data.getUnscaledValue(name).set(((Number) daten).longValue());
			}
		} else if (daten instanceof LogischerWert) {
			final boolean wert = ((LogischerWert) daten).getBoolWert();
			if (wert) {
				data.getUnscaledValue(name).set(1);
			} else {
				data.getUnscaledValue(name).set(0);
			}
		} else if (daten instanceof Boolean) {
			final boolean wert = (Boolean) daten;
			if (wert) {
				data.getUnscaledValue(name).set(1);
			} else {
				data.getUnscaledValue(name).set(0);
			}
		} else {
			data.getTextValue(name).setText(daten.toString());
		}
	}
}
