/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.trend;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Ein Dateneintrag innerhalb eines Trendobjekts.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TrendEintrag implements Kopierbar {

	/**
	 * der Zeitstempel des Eintrags.
	 */
	private final Zeitstempel zeitstempel;

	/**
	 * der Wert des Eintrags.
	 */
	private final Number wert;

	/**
	 * Konstruktor mit Angabe von Wert und Zeitstempel.<br>
	 *
	 * @param zeit
	 *            der Zeitstempel
	 * @param wert
	 *            der Wert
	 */
	public TrendEintrag(final Zeitstempel zeit, final Number wert) {
		if ((zeit == null) || (wert == null)) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}
		this.zeitstempel = zeit;
		this.wert = wert;
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public TrendEintrag erzeugeKopie() {
		return new TrendEintrag(zeitstempel.erzeugeKopie(), wert);
	}

	/**
	 * ermittelt, ob der übergebene Eintrag bezüglich des aktuellen monoton
	 * fallend ist.
	 *
	 * @param o
	 *            der Vergleichswert
	 * @return wahr, wenn der übergebene Eintrag streng monoton fallend ist.
	 */
	public LogischerWert fallend(final TrendEintrag o) {
		return LogischerWert.valueOf(wert.doubleValue() > o.getWert().doubleValue());
	}

	/**
	 * liefert den Wert des Eintrags.
	 *
	 * @return den Wert
	 */
	public Number getWert() {
		return wert;
	}

	/**
	 * liefert den Zeitstempel des Eintrags als Wert in Millisekunden.
	 *
	 * @return der Zeitstempel
	 */
	public long getZeit() {
		return zeitstempel.getMilliSekunden();
	}

	/**
	 * ermittelt, ob der übergebene Eintrag bezüglich des aktuellen konstant
	 * ist.
	 *
	 * @param o
	 *            der Vergleichswert
	 * @return wahr, wenn der übergebene Eintrag monoton steigend ist.
	 */
	public LogischerWert konstant(final TrendEintrag o) {
		return LogischerWert.valueOf(wert.doubleValue() == o.getWert().doubleValue());
	}

	/**
	 * ermittelt, ob der übergebene Eintrag bezüglich des aktuellen monoton
	 * steigend ist.
	 *
	 * @param o
	 *            der Vergleichswert
	 * @return wahr, wenn der übergebene Eintrag streng monoton steigend ist.
	 */
	public LogischerWert steigend(final TrendEintrag o) {
		return LogischerWert.valueOf(wert.doubleValue() < o.getWert().doubleValue());
	}

	/**
	 * liefert die Repräsentation des Eintrags als Zeichenkette.<br>
	 * Die Ausgabe erfolgt dabei in der Form "(&lt;zeit&gt;:&lt;wert&gt;)".
	 *
	 * @return die Zeichenkette
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "(" + zeitstempel + ": " + wert + ")";
	}
}
