/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.verwaltung;

import java.io.File;
import java.security.interfaces.RSAPublicKey;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.uda.UdaMessage;
import de.bsvrz.ibv.uda.uda.UmfassendeDatenanalyse;
import de.bsvrz.ibv.uda.uda.data.Modul;
import de.bsvrz.ibv.uda.uda.data.Skript;
import de.bsvrz.ibv.uda.uda.data.UDA;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.dynobj.DynObjektException;
import de.bsvrz.sys.funclib.dynobj.DynamischeObjekte;

/**
 * Eine Klasse zur Repräsentation eines UdaModuls in der "Umfassenden
 * Datenanalyse".<br>
 * Ein UdaModul repräsentiert eine Sammlung von Skripten und den zugehörigen
 * Skriptläufen. Eine Instanz dieser Klasse hat eine Verbindung zum
 * Datenverteiler und empfängt und publiziert den Status und Änderungen für
 * Skripte und Skriptläufe.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ServerUdaModul extends Modul implements MutableSetChangeListener, ClientSenderInterface {

	private static final Debug LOGGER2 = Debug.getLogger();

	/**
	 * der Logger für das UdaModul.
	 */
	private final Debug logger = ServerUdaModul.LOGGER2;

	/**
	 * die Liste der für das Modul definierten Skripte.
	 */
	private final Map<SystemObject, ServerSkript> skriptListe = new HashMap<>();

	/**
	 * die Liste der für das Modul definierten Skriptläufe.
	 */
	private final Map<SystemObject, ServerSkriptLauf> skriptLaufListe = new HashMap<>();

	/**
	 * die verwendete Datenverteilerverbindung.
	 */
	private final UmfassendeDatenanalyse uda;

	/**
	 * das Wurzelverzeichnis für die persitente Speicherung der UDA-Daten.
	 */
	private final File modulVerzeichnis;

	/**
	 * die Menge innerhalb der Datenverteilerkonfiguration, in der die Skripte
	 * des Moduls abgelegt sind.
	 */
	private final MutableSet skriptMenge;

	/**
	 * die Menge innerhalb der Datenverteilerkonfiguration, in der die
	 * Skriptläufe des Moduls abgelegt sind.
	 */
	private final MutableSet laufMenge;

	/**
	 * die Schlüssel für die verschlüsselte Übertragung des Passworts für die
	 * Anmeldung eines Nutzers bei der Ausführung eines Skriptlaufs.
	 */
	private final UdaSchluessel schluessel;

	/**
	 * die Datenbeschreibung für die Veröffentlichung des öffentlichen
	 * Schlüssels.
	 */
	private final DataDescription keyDesc;

	/**
	 * Konstruktor.<br>
	 * Die Funktion realisiert die serverseitige Implementierung eines
	 * UdaModuls, welches eine Menge von Skripten und Skriptläufen verwaltet und
	 * diese zur Ausführung bringt.
	 *
	 * @param uda
	 *            die Datenverteilerschnittstelle
	 * @param object
	 *            das Systemobjekt
	 * @throws OneSubscriptionPerSendData
	 *             es ist bereits eine andere Anwendung als Quelle für die Daten
	 *             angemeldet, die von diesem Modul versendet werden sollen
	 * @throws UdaServerException
	 *             es ist ein Fehler bei der Initialisierung des Modulobjekts
	 *             aufgetreten
	 */
	public ServerUdaModul(final UmfassendeDatenanalyse uda, final ConfigurationObject object)
			throws OneSubscriptionPerSendData, UdaServerException {
		super(uda.getVerbindung(), object);
		this.uda = uda;

		schluessel = new UdaSchluessel();

		modulVerzeichnis = new File(uda.getUdaModulRoot(), object.getPid());
		if (!modulVerzeichnis.exists()) {
			modulVerzeichnis.mkdirs();
		}

		keyDesc = new DataDescription(uda.getVerbindung().getDataModel().getAttributeGroup(UDA.MODUL_ATG_KEY),
				uda.getVerbindung().getDataModel().getAspect(UDA.ASP_IST));

		uda.getVerbindung().subscribeSender(this, object, keyDesc, SenderRole.source());

		skriptMenge = object.getMutableSet(UDA.MODUL_MENGE_SKRIPTE);
		update(skriptMenge, skriptMenge.getElements().toArray(new SystemObject[0]), new SystemObject[0]);
		skriptMenge.addChangeListener(this);

		laufMenge = object.getMutableSet(UDA.MODUL_MENGE_LAEUFE);
		update(laufMenge, laufMenge.getElements().toArray(new SystemObject[0]), new SystemObject[0]);
		laufMenge.addChangeListener(this);

		loadSkriptDaten();

		uda.getStartTimer().schedule(new SkriptAusfuehrer(this), 1000L, 1000L);

		ServerUdaModul.LOGGER2.info("Warte auf Daten für UdaModul: " + object.getNameOrPidOrId());
	}

	/**
	 * legt für neue in die Menge der Skriptläufe eingetragene Systemobjekte die
	 * entsprechenden Skriptlaufobjekte innerhalb des Moduls an.
	 *
	 * @param added
	 *            die Menge der hinzugefügten Systemobjekte
	 */
	private void addLaufe(final SystemObject[] added) {
		for (final SystemObject obj : added) {
			if (obj.isOfType(UDA.TYP_LAUF)) {
				if (!skriptListe.containsKey(obj)) {
					try {
						skriptLaufListe.put(obj, new ServerSkriptLauf(this, obj));
					} catch (final UdaServerException e) {
						LogTools.log(getLogger(), UdaMessage.ErrorSkriptlaufAnlegen, obj.getNameOrPidOrId(),
								e.getMessage());
					}
				}
			}
		}
	}

	/**
	 * legt für neue in die Menge der Skripte eingetragene Systemobjekte die
	 * entsprechenden Skriptobjekte innerhalb des Moduls an.
	 *
	 * @param added
	 *            die Menge der hinzugefügten Systemobjekte
	 */
	private void addSkripte(final SystemObject[] added) {
		for (final SystemObject obj : added) {
			if (obj.isOfType(UDA.TYP_SKRIPT)) {
				if (!skriptListe.containsKey(obj)) {
					try {
						skriptListe.put(obj, new ServerSkript(this, obj));
					} catch (final UdaServerException e) {
						LogTools.log(getLogger(), UdaMessage.ErrorSkriptAnlegen, obj.getNameOrPidOrId(),
								e.getMessage());
					}
				}
			}
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion implementiert die entsprechende Funktion der Schnittstelle
	 * {@link ClientSenderInterface}. Das Objekt stellt als Quelle den
	 * öffentlichen Schlüssel für die Verschlüsselung des Passworts bei der
	 * Übertraging zur Verfügung.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientSenderInterface#dataRequest(de.bsvrz.dav.daf.main.config.SystemObject,
	 *      de.bsvrz.dav.daf.main.DataDescription, byte)
	 */
	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		if (state == ClientSenderInterface.START_SENDING) {
			if (dataDescription.getAttributeGroup().equals(keyDesc.getAttributeGroup())) {
				if (dataDescription.getAspect().equals(keyDesc.getAspect())) {
					sendKeyData();
				}
			}
		}
	}

	/**
	 * die Funktion entschlüsselt ein übergebenes verschlüsseltes Passwort mit
	 * dem dynamisch angelegten privaten Schlüssel.
	 *
	 * @param data
	 *            das verschlüsselte Passwort
	 * @return das entschlüsselte Passwort
	 * @throws UdaServerException
	 *             das passwort konnte nicht entschlüsselt werden
	 */
	String decrypt(final byte[] data) throws UdaServerException {
		if (data.length <= 0) {
			return Constants.EMPTY_STRING;
		}
		return schluessel.decrypt(data);
	}

	/**
	 * liefert den algemein zu verwendenden Logger für das Modul.
	 *
	 * @return den Logger
	 */
	public Debug getLogger() {
		return logger;
	}

	/**
	 * liefert das Verzeichnis, in dem die Daten des Moduls abgelegt werden.
	 *
	 * @return das Verzeichnis
	 */
	public File getModulVerzeichnis() {
		return modulVerzeichnis;
	}

	/**
	 * ermittelt das Skript innerhalb des Moduls, das innerhalb der
	 * Konfiguration des Datenverteilers durch ein Objekt mit der angegebenen
	 * Pid repräsentiert wird. {@inheritDoc}
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.Modul#getSkript(java.lang.String)
	 */
	@Override
	public Skript getSkript(final String pid) {
		Skript result = null;
		for (final ServerSkript skript : skriptListe.values()) {
			if ((skript.getDavObjekt() != null) && (skript.getDavObjekt().getPid().equals(pid))) {
				result = skript;
				break;
			}
		}
		return result;
	}

	/**
	 * die Funktion liefert das Skript, das innerhalb des Moduls durch das
	 * übergebene Systemobjekt repräsentiert wird.
	 *
	 * @param object
	 *            das Systemobjekt
	 * @return das Skript oder <i>null</i>, wenn kein entsprechendes Skript
	 *         gefunden werden konnte.
	 */
	ServerSkript getSkript(final SystemObject object) {
		ServerSkript result = null;
		for (final ServerSkript skript : skriptListe.values()) {
			if (skript.getDavObjekt().equals(object)) {
				result = skript;
				break;
			}
		}
		return result;
	}

	/**
	 * liefert die Liste der verwalteten Skriptläufe.<br>
	 * Die Liste wird vom Skriptausführermodul überwacht, um Skriptläufe in der
	 * definierten Art und Weise zur Ausführung zu bringen.
	 *
	 * @return die Liste
	 */
	Collection<ServerSkriptLauf> getSkriptLaufListe() {
		return skriptLaufListe.values();
	}

	/**
	 * ermittelt das Skript mit dem angegebenen Name.
	 *
	 * @param name
	 *            der Name des gesuchten Skripts.
	 * @return das Skript
	 */
	public Skript getSkriptWithName(final String name) {
		Skript result = null;
		for (final ServerSkript skript : skriptListe.values()) {
			if (skript.getName().equals(name)) {
				result = skript;
				break;
			}
		}
		return result;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion implementiert die entsprechende Funktion der Schnittstelle
	 * {@link ClientSenderInterface}. Die Sendesteuerung wird für alle
	 * angemeldeten Datenspezifikationen unterstützt.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientSenderInterface#isRequestSupported(de.bsvrz.dav.daf.main.config.SystemObject,
	 *      de.bsvrz.dav.daf.main.DataDescription)
	 */
	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return true;
	}

	/**
	 * die Funktion lädt die im Filesystem des Servers abgelegten Skriptdaten.
	 * <br>
	 * Das Verzeichnis wird durchlaufen und alle Skriptobjekte und
	 * Skriptlaufobjekte angelegt, die noch nicht innerhalb der
	 * Datenverteilerkonfiguration vorhanden sind. Das Einlesen der eigentlichen
	 * Daten wird durch die konkreten Objekte für Skripte und Skriptläufe
	 * ausgeführt.
	 */
	private void loadSkriptDaten() {
		final DataModel model = uda.getVerbindung().getDataModel();

		File[] skripts = new File(modulVerzeichnis, "skripte").listFiles();

		if (skripts != null) {
			for (final File item : skripts) {
				final String pid = item.getName();
				if (pid.startsWith(".")) {
					continue;
				}
				SystemObject obj = model.getObject(pid);
				try {
					if (obj == null) {
						final DynamicObjectType type = (DynamicObjectType) model.getType(UDA.TYP_SKRIPT);
						obj = DynamischeObjekte.getInstanz(uda.getVerbindung()).erzeugeObjekt(type, pid, pid);
					}
					if (obj != null) {
						if (!skriptMenge.getElements().contains(obj)) {
							skriptMenge.add(obj);
						}
					}
				} catch (final ConfigurationChangeException | DynObjektException e) {
					throw new RuntimeException("Dynamisches Objekt für ein Skript konnte nicht angelegt werden: "
							+ e.getMessage(), e);
				}
			}
		}

		skripts = new File(modulVerzeichnis, "lauf").listFiles();

		if (skripts != null) {
			for (final File item : skripts) {
				final String pid = item.getName();
				if (pid.startsWith(".")) {
					continue;
				}
				SystemObject obj = model.getObject(pid);
				try {
					if (obj == null) {
						final DynamicObjectType type = (DynamicObjectType) model.getType(UDA.TYP_LAUF);
						obj = DynamischeObjekte.getInstanz(uda.getVerbindung()).erzeugeObjekt(type, pid, pid);
					}
					if (obj != null) {
						if (!laufMenge.getElements().contains(obj)) {
							laufMenge.add(obj);
						}
					}
				} catch (final ConfigurationChangeException | DynObjektException e) {
					throw new RuntimeException("Dynamisches Objekt für einen Skriptlauf konnte nicht angelegt werden: "
							+ e.getMessage(), e);
				}
			}
		}
	}

	/**
	 * entfernt die Skriptlaufobjekte, die durch die Menge der übergebenen
	 * Systemobjekte repräsentiert werden.
	 *
	 * @param removed
	 *            die Menge der entfernten Systemobjekte
	 */
	private void removeLaufe(final SystemObject[] removed) {
		for (final SystemObject obj : removed) {
			if (obj.isOfType(UDA.TYP_LAUF)) {
				final ServerSkriptLauf lauf = skriptLaufListe.get(obj);
				skriptLaufListe.remove(obj);
				if (lauf != null) {
					try {
						lauf.remove();
					} catch (final ConfigurationChangeException e) {
						throw new RuntimeException("Konfiguration kann nicht geändert werden", e);
					}
				}
			}
		}
	}

	/**
	 * entfernt die Skriptobjekte, die durch die Menge der übergebenen
	 * Systemobjekte repräsentiert werden.
	 *
	 * @param removed
	 *            die Menge der entfernten Systemobjekte
	 */
	private void removeSkripte(final SystemObject[] removed) {
		for (final SystemObject obj : removed) {
			if (obj.isOfType(UDA.TYP_SKRIPT)) {
				final ServerSkript skript = skriptListe.get(obj);
				skriptListe.remove(obj);
				if (skript != null) {
					try {
						skript.remove();
					} catch (final ConfigurationChangeException e) {
						throw new RuntimeException("Konfiguration kann nicht geändert werden", e);
					}
				}
			}
		}
	}

	/**
	 * die Funktion versendet den öffentlichen Schlüssel für die gesicherte
	 * Übertragung von Passworten.
	 */
	private void sendKeyData() {
		final Data data = uda.getVerbindung().createData(keyDesc.getAttributeGroup());
		final RSAPublicKey publicKey = schluessel.getPublic();

		byte[] keyArray = publicKey.getModulus().toByteArray();
		Data.Array dataArray = data.getArray("Modulus");
		dataArray.setLength(keyArray.length);
		for (int idx = 0; idx < keyArray.length; idx++) {
			dataArray.getItem(idx).asUnscaledValue().set(keyArray[idx]);
		}

		keyArray = publicKey.getPublicExponent().toByteArray();
		dataArray = data.getArray("Exponent");
		dataArray.setLength(keyArray.length);
		for (int idx = 0; idx < keyArray.length; idx++) {
			dataArray.getItem(idx).asUnscaledValue().set(keyArray[idx]);
		}

		try {
			uda.getVerbindung().sendData(new ResultData(getModulObjekt(), keyDesc, System.currentTimeMillis(), data));
		} catch (final SendSubscriptionNotConfirmed e) {
			ServerUdaModul.LOGGER2.error(e.getLocalizedMessage());
			// sollte nicht auftreten, da erst nach Sendeanforderung gesendet
			// wird
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion beobachtet die Mengen der Skripte und der Skriptläufe
	 * innerhalb des Uda-Moduls. Bei Änderungen in den Mengen werden die
	 * entsprechenden Objekte zur Repräsentation von Skripten und Skriptläufen
	 * angelegt oder entfernt. Die Publizierung der Daten übernehmen die Objekte
	 * selbst.
	 *
	 * @see de.bsvrz.dav.daf.main.config.MutableSetChangeListener#update(de.bsvrz.dav.daf.main.config.MutableSet,
	 *      de.bsvrz.dav.daf.main.config.SystemObject[],
	 *      de.bsvrz.dav.daf.main.config.SystemObject[])
	 */
	@Override
	public void update(final MutableSet liste, final SystemObject[] added, final SystemObject[] removed) {

		if (liste.equals(skriptMenge)) {
			addSkripte(added);
			removeSkripte(removed);
		}
		if (liste.equals(laufMenge)) {
			addLaufe(added);
			removeLaufe(removed);
		}

		for (final SystemObject obj : removed) {
			if (obj.isOfType(UDA.TYP_SKRIPT)) {
				final ServerSkript skript = skriptListe.get(obj);
				skriptListe.remove(obj);
				if (skript != null) {
					try {
						skript.remove();
					} catch (final ConfigurationChangeException e) {
						throw new RuntimeException("Konfiguration kann nicht geändert werden", e);
					}
				}
			} else if (obj.isOfType(UDA.TYP_LAUF)) {
				skriptLaufListe.remove(obj);
				final ServerSkriptLauf lauf = skriptLaufListe.get(obj);
				if (lauf != null) {
					try {
						lauf.remove();
					} catch (final ConfigurationChangeException e) {
						throw new RuntimeException("Konfiguration kann nicht geändert werden", e);
					}
				}
			}
		}
	}
}
